// ==================================================================================================
// ku2zoff: kluch gazoviy MDL by Killsniper.   .    
// ==================================================================================================
#include "extdll.h"
#include "util.h"
#include "cbase.h"
#include "monsters.h"
#include "weapons.h"
#include "nodes.h"
#include "player.h"
#include "gamerules.h"
#include "decals.h"

void FindHullIntersection(const Vector &vecSrc, TraceResult &tr, float *mins, float *maxs, edict_t *pEntity);

#define WRENCH_WEIGHT		0
#define WRENCH_DMG RANDOM_FLOAT(65,70)
#define WRENCH_DMG_BIG RANDOM_FLOAT(150,155)
#define WRENCH_DISTANCE 48

LINK_ENTITY_TO_CLASS( weapon_wrench, CWPNWrench );

enum wrench_e {
	WRENCH_DRAW = 0,
	WRENCH_IDLE1,
	WRENCH_ATTACK1HIT,
	WRENCH_ATTACK1MISS,
	WRENCH_ATTACK2HIT,
	WRENCH_ATTACK2MISS,
	WRENCH_ATTACK3HIT,
	WRENCH_ATTACK3MISS,
	WRENCH_CHARGE,
	WRENCH_HIT,
	WRENCH_MISS,
	WRENCH_HOLSTER
};

void CWPNWrench::Spawn( )
{
	Precache( );
	m_iId = WEAPON_WRENCH;
	SET_MODEL(ENT(pev), "models/w_wrench.mdl");
	m_iClip = -1;

	pev->sequence = 1;

	FallInit();
}

void CWPNWrench::Precache( void )
{
	PRECACHE_MODEL("models/v_wrench.mdl");
	PRECACHE_MODEL("models/w_wrench.mdl");
	PRECACHE_SOUND("weapons/wrench_hitwall1.wav");
	PRECACHE_SOUND("weapons/wrench_hitwall2.wav");
	PRECACHE_SOUND("weapons/wrench_hitwall_big1.wav");
	PRECACHE_SOUND("weapons/wrench_hitwall_big2.wav");
	PRECACHE_SOUND("weapons/wrench_hitbod1.wav");
	PRECACHE_SOUND("weapons/wrench_hitbod2.wav");
	PRECACHE_SOUND("weapons/wrench_hitbod3.wav");
	PRECACHE_SOUND("weapons/wrench_hitbod_big1.wav");
	PRECACHE_SOUND("weapons/wrench_hitbod_big2.wav");
	PRECACHE_SOUND("weapons/wrench_miss1.wav");
	PRECACHE_SOUND("weapons/wrench_miss2.wav");
	PRECACHE_SOUND("weapons/wrench_miss_big.wav");

	m_usWrenchAttack = PRECACHE_EVENT ( 1, "events/wpn_wrench.sc" );
}

int CWPNWrench::GetItemInfo(ItemInfo *p)
{
	p->pszName = STRING(pev->classname);
	p->pszAmmo1 = NULL;
	p->iMaxAmmo1 = -1;
	p->pszAmmo2 = NULL;
	p->iMaxAmmo2 = -1;
	p->iMaxClip = WEAPON_NOCLIP;
	p->iSlot = 0;
	p->iPosition = 3;
	p->iId = WEAPON_WRENCH;
	p->iWeight = WRENCH_WEIGHT;
	return 1;
}

int CWPNWrench::AddToPlayer( CBasePlayer *pPlayer )
{
	if ( CBasePlayerWeapon::AddToPlayer( pPlayer ) )
	{
		MESSAGE_BEGIN( MSG_ONE, gmsgWeapPickup, NULL, pPlayer->pev );
			WRITE_BYTE( m_iId );
		MESSAGE_END();
		return TRUE;
	}
	return FALSE;
}

BOOL CWPNWrench::Deploy( )
{
	return DefaultDeploy( "models/v_wrench.mdl", "models/w_wrench.mdl", WRENCH_DRAW, "crowbar" );
}

void CWPNWrench::Holster( int skiplocal /* = 0 */ )
{
	SendWeaponAnim(WRENCH_HOLSTER);
	m_flSwingTime = m_flSwingRelease = 0;
	m_pPlayer->m_flNextAttack = UTIL_WeaponTimeBase() + 0.4;
}

void CWPNWrench::PrimaryAttack()
{
	if ( m_flSwingTime ) return;

	Attack(FALSE);
	m_flTimeWeaponIdle = UTIL_WeaponTimeBase() + 1.0;
}

void CWPNWrench::SecondaryAttack()
{
	if ( !m_flSwingTime )
	{
		m_flSwingTime = gpGlobals->time;
		m_flSwingRelease = 0;

		PLAYBACK_EVENT_FULL(0, m_pPlayer->edict(), m_usWrenchAttack, 0.0, (float *)&g_vecZero, (float *)&g_vecZero,
		0.0, 0.0, 0, 2, FALSE, FALSE );

		m_flTimeWeaponIdle = m_flNextPrimaryAttack = m_flNextSecondaryAttack = UTIL_WeaponTimeBase() + 0.9;
	}
}

void CWPNWrench::Attack( bool fSecAttack )
{
	TraceResult tr;

	UTIL_MakeVectors (m_pPlayer->pev->v_angle);
	Vector vecSrc	= m_pPlayer->GetGunPosition( );
	Vector vecEnd = vecSrc + gpGlobals->v_forward * WRENCH_DISTANCE;

	UTIL_TraceLine( vecSrc, vecEnd, dont_ignore_monsters, ENT( m_pPlayer->pev ), &tr );

#ifndef CLIENT_DLL
	if ( tr.flFraction >= 1.0 )
	{
		UTIL_TraceHull( vecSrc, vecEnd, dont_ignore_monsters, head_hull, ENT( m_pPlayer->pev ), &tr );
		if ( tr.flFraction < 1.0 )
		{
			CBaseEntity *pHit = CBaseEntity::Instance( tr.pHit );

			if ( !pHit || pHit->IsBSPModel() )
				FindHullIntersection( vecSrc, tr, VEC_DUCK_HULL_MIN, VEC_DUCK_HULL_MAX, m_pPlayer->edict() );

			vecEnd = tr.vecEndPos;
		}
	}
#endif
	m_pPlayer->m_iWeaponVolume = NORMAL_GUN_VOLUME;
	m_pPlayer->SetAnimation( PLAYER_ATTACK1 );

	//      
	int iHit;
	int iHitWorld;
	float flDelay = 1;

	if (tr.flFraction >= 1.0) // 
	{
		iHit = FALSE;
		iHitWorld = FALSE;
		if (fSecAttack)	flDelay = 0.5;
		else	flDelay = 0.7;
	}
	else // 
	{
		iHit = TRUE;
		iHitWorld = TRUE;

#ifndef CLIENT_DLL
		CBaseEntity *pEntity = CBaseEntity::Instance(tr.pHit);

		//         
		int bitsDamage;

		if (pEntity->IsAlive())
			bitsDamage = DMG_CLUB | DMG_NEVERGIB;
		else
			bitsDamage = DMG_CLUB;

		ClearMultiDamage( );
		if (fSecAttack) // secondary attack
			pEntity->TraceAttack(m_pPlayer->pev, WRENCH_DMG_BIG, gpGlobals->v_forward, &tr, bitsDamage);
		else // primary attack
			pEntity->TraceAttack(m_pPlayer->pev, WRENCH_DMG, gpGlobals->v_forward, &tr, bitsDamage);
		ApplyMultiDamage( m_pPlayer->pev, m_pPlayer->pev );

		if (pEntity)
		{
			if ( pEntity->Classify() != CLASS_NONE && pEntity->Classify() != CLASS_MACHINE )
			{
				iHitWorld = FALSE;
			}
		}
#endif		
		if (fSecAttack)	flDelay = 0.25;
		else	flDelay = 0.5;
	}
	
	PLAYBACK_EVENT_FULL(0, m_pPlayer->edict(), m_usWrenchAttack, 0.0, (float *)&g_vecZero, (float *)&g_vecZero,
	0.0, 0.0, 0, fSecAttack, iHit, iHitWorld );

	if (iHit && iHitWorld)
	{
		if (fSecAttack)
			UTIL_DecalTrace(&tr, DECAL_GARGSTOMP1);
		else
			UTIL_DecalTrace(&tr, DECAL_GUNSHOT1 + RANDOM_LONG(0, 4));
	}
	
	m_flNextPrimaryAttack = m_flNextSecondaryAttack = UTIL_WeaponTimeBase() + flDelay;
}

void CWPNWrench::WeaponIdle( void )
{
	if ( m_flSwingRelease == 0 && m_flSwingTime )
		 m_flSwingRelease = gpGlobals->time;

	if ( m_flTimeWeaponIdle > UTIL_WeaponTimeBase() )
		return;

	if ( m_flSwingTime )
	{
		Attack(TRUE);
					
        m_flSwingRelease = 0;
		m_flSwingTime = 0;
		m_flNextSecondaryAttack = m_flNextPrimaryAttack = UTIL_WeaponTimeBase() + 1.3;
		m_flTimeWeaponIdle = UTIL_WeaponTimeBase() + 1.7;
			
		return;
	}
	else if ( m_flSwingRelease > 0 )
	{
		m_flSwingTime = 0;
			
		RetireWeapon();
		return;
		
		m_flTimeWeaponIdle = UTIL_WeaponTimeBase() + UTIL_SharedRandomFloat( m_pPlayer->random_seed, 10, 15 );
		m_flSwingRelease = -1;
		return;
	}

	m_flTimeWeaponIdle = UTIL_WeaponTimeBase() + UTIL_SharedRandomFloat( m_pPlayer->random_seed, 10, 15 );
	SendWeaponAnim( WRENCH_IDLE1 );
}